% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $
%
classdef XFLT
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here
    
   properties%(SetAccess='private')
    MY=[];  %column vector
    MOmega=[]; %column vector
    MX=[];
    q=[]; %approximation rank
    epsilon=[];%target accuracy
    
    %maximal block size
    blocksize=[];
        
    Ftype=[]; %Type of XFLT-version

    fhandle=[]; %function handle for special FFT versions
    
    %NFFT library directory
    XFFTdir=[];
    
    %plan for NFFT or Butterfly
    XFFT=[]

    %for FFT, NFFT and Butterfly
    N=[]; 

    %only for Butterfly
    p=[];
    
    
    
  end%properties

properties%(Hidden=true,SetAccess='private',GetAccess='private')
    %parameters for the Laplace part
    M=[];%number of boxes
    
    maxy=[];%maximal value in Y
    maxxi=[];%maximal value in Omega
    
    Y=[];%decomposition of MY -> [Box Index, first node index , last node index]
    Omega=[];%decomposition MOmega-> [Box Index, first node index , last node index]
    
    l=[];%lower bound in approximation case three
    m=[];%upper bound in approximation case three
    
    Omega_num=[];%number of boxes in Omega
    Kend=[];%number of boxes in Omega -1 in case of last box Omega_M is nonempty
    
    Y_num=[];%number of boxes in Y
    Iend=[];%numver of boxes in Y -1 in case of last box Y_M is nonempty
   
    
    t=[]; %chebyshev nodes
    lambda=[]; %w'(t_s) without constant factors
    
    %parameters for the XFFT part
    
    
end%properties

methods
    function plan=XFLT(Ftype,MY,option,param,varargin)
        
        plan.Ftype=Ftype;
        plan.MY=MY;
        
          
        
        
        switch Ftype
            
            case 'LT'
                                
                nopt=size(varargin,2);
                if(mod(nopt,2)~=0)
                    error('Wrong number of varargins. The number should be even because for each property there are two fields (name and value).');
                else
		
                    for k=1:nopt/2
                        prop=varargin{2*k-1};
                        val=varargin{2*k};
                        switch prop
                            case 'MOmega'
                                plan.MOmega=val;
                            case 'blocksize'
                                plan.blocksize=val;
                            otherwise
                            error('Unknown option ''%s''.',prop);                      
                        
                        end %switch
                    end %for
                end %if
                              
            case 'FFLT'

                plan.N=size(MY,1);
                plan.MOmega=(0:plan.N-1)';                           
                plan.fhandle=@(plan,fhat) mtimes_FFT(plan,fhat);
                plan.MX=plan.MOmega; %just for exact evaluation
                nopt=size(varargin,2);
                if(mod(nopt,2)~=0)
                    error('Wrong number of varargins. The number should be even because for each property there are two fields (name and value).');
                else
                    for k=1:nopt/2
                        prop=varargin{2*k-1};
                        val=varargin{2*k};
                        switch prop
                            case 'blocksize'
                                plan.blocksize=val;
                            otherwise
                            error('Unknown option ''%s''.',prop);                            
                        end
                    end%for
                end%if
              
            case 'NFFLT'
                
                nopt=size(varargin,2);
                if(mod(nopt,2)~=0)
                    error('Wrong number of varargins. The number should be even because for each property there are two fields (name and value).');
                else
		
                    for k=1:nopt/2
                        prop=varargin{2*k-1};
                        val=varargin{2*k};
                        switch prop
                            case 'MX'
                                plan.MX=val;
                            case 'libdir'
                                plan.XFFTdir=val;
                            case 'N'
                                plan.N=val;
                            case 'blocksize'
                                plan.blocksize=val;
                            otherwise
                            error('Unknown option ''%s''.',prop);                      
                        
                        end %switch
                    end %for
                    
                end %if
                plan.fhandle=@(plan,fhat) mtimes_NFFT(plan,fhat);
                addpath(strcat(plan.XFFTdir,'/matlab/nfft/'));
                addpath(strcat(plan.XFFTdir,'/matlab/nnfft/'));
                
                
                M1=size(plan.MX,1);
                %plan.MOmega=(0:1:plan.N-1).';
                plan.MOmega=(1:1:plan.N).';
%                 %initialisation of plan for the nfft
%                 n=2^(ceil(log(max(plan.N))/log(2))+1);
%                 plan.XFFT=nfft(1,plan.N,M1,n,n,plan.q,'PRE_PHI_HUT','FFTW_MEASURE'); % use of nfft_init_guru
%                 %plan.XFFT=nfft(1,plan.N,M1);
%                 plan.XFFT.x=-plan.MX/plan.N;
%                 
%                 nfft_precompute_psi(plan.XFFT);
%                 
%                 plan.fhandle=@(plan,fhat) mtimes_NFFT(plan,fhat);
                
            case 'BSFFLT'
                nopt=size(varargin,2);
                if(mod(nopt,2)~=0)
                    error('Wrong number of varargins. The number should be even because for each property there are two fields (name and value).');
                else
		
                    for k=1:nopt/2
                        prop=varargin{2*k-1};
                        val=varargin{2*k};
                        switch prop
                            case 'MX'
                                plan.MX=val;
                            case 'MOmega'
                                plan.MOmega=val;                           
                            case 'N'
                                plan.N=val; 
                            case 'BSFFTrank'
                                plan.p=val;
                            case 'blocksize'
                                plan.blocksize=val;
                            otherwise
                            error('Unknown option ''%s''.',prop);                      
                        
                        end %switch
                    end %for
                end %if

                plan.XFFT=sparse_FFT1D(plan.MX,plan.MOmega,plan.p,plan.N,'Ltb','no_copinfo');
                plan.fhandle=@(plan,fhat) mtimes_BSFFT(plan,fhat);
                
                
                case 'NNFFLT'
                nopt=size(varargin,2);
                if(mod(nopt,2)~=0)
                    error('Wrong number of varargins. The number should be even because for each property there are two fields (name and value).');
                else
		
                    for k=1:nopt/2
                        prop=varargin{2*k-1};
                        val=varargin{2*k};
                        switch prop
                            case 'MX'
                                plan.MX=val;
                            case 'MOmega'
                                plan.MOmega=val;
                            case 'libdir'
                                plan.XFFTdir=val;
                            case 'N'
                                plan.N=val; 
                            case 'blocksize'
                                plan.blocksize=val;
                            otherwise
                            error('Unknown option ''%s''.',prop);                      
                        
                        end %switch
                    end %for
                end %if
                
                plan.fhandle=@(plan,fhat) mtimes_NNFFT(plan,fhat);
                addpath(genpath(plan.XFFTdir));
                
                
          
                
            otherwise
                error('Unknown type ''%s''.',Ftype);
        end


        if(~plan.MOmega)
            error('You have to set MOmega');
        else
        plan.maxy=max(MY);
        plan.maxxi=max(plan.MOmega);

            switch option
                case 'rank'
                    plan.q=param;
                    plan.epsilon=4^(-plan.q);           
                    plan.M=ceil( (plan.q*log(4)+log(plan.maxy*plan.maxxi) )/log(2)+1); %number of boxes in dependence of rank q
                case 'accuracy'
                    plan.epsilon=param;
                    plan.M=ceil(log2(plan.maxy*plan.maxxi/plan.epsilon))+1; %number of boxes in dependence of target accuracy epsilon
                    plan.q=ceil(log(1/plan.epsilon)/log(4));           
                otherwise
                    display('unknown option')
            end
        end%


        switch Ftype
             case 'NFFLT'
                plan=precomputations(plan);
                %initialisation of plan for the nfft
                n=2^(ceil(log(max(plan.N))/log(2))+1);
                r=ceil(plan.q/3); %approximation rank of the NFFT
               % plan.XFFT=nnfft(1,plan.N,M1,n,r,'PRE_PHI_HUT','FFTW_MEASURE'); % use of nfft_init_guru
                plan.XFFT=nfft(1,plan.N,M1,n,r,'PRE_PHI_HUT','FFTW_MEASURE'); % use of nfft_init_guru
                
                
                
                %plan.XFFT=nfft(1,plan.N,M1);
                plan.XFFT.x=-plan.MX/plan.N;
                nfft_precompute_psi(plan.XFFT);
                
                 %check if a library directory is set
                if(strcmp(plan.Ftype,'NFFLT') || strcmp(plan.Ftype,'BSFFLT') ) 
                    if(~plan.XFFTdir)
                        error('You have to set a library directory in the property libdir')
                        else plan=precomputations(plan);
                    end%if
                end%if
                
              case 'NNFFLT'
                plan=precomputations(plan);
                %initialisation of plan for the nfft
                M2=size(plan.MOmega,1);
                M1=size(plan.MX,1);
                
                n=2^(ceil(log(max(plan.N))/log(2))+1);
                %r=ceil(plan.q/3); %approximation rank of the NFFT
                r=plan.q;
                %plan.XFFT=nnfft_init(1,M2,M1,plan.N,n,r,'PRE_PHI_HUT','FFTW_MEASURE'); % use of nfft_init_guru
                %plan.XFFT=nnfft_init(1,M2,M1,plan.N);
                plan.XFFT=nnfft_init_guru(1,M2,M1,plan.N,n,r,bitor(PRE_PSI,PRE_PHI_HUT));
               %plan.XFFT=nnfft(1,M2,M1,plan.N,n,r,'PRE_PHI_HUT');
                
                %plan.XFFT=nfft(1,plan.N,M1);
                nnfft_set_x(plan.XFFT,(-plan.MX/plan.N+1/2).');
                nnfft_set_v(plan.XFFT,(plan.MOmega/plan.N-1/2).');
                %plan.XFFT.x=-plan.MX/plan.N;
                nnfft_precompute_psi(plan.XFFT);
                
                 %check if a library directory is set
                if(strcmp(plan.Ftype,'NFFLT') || strcmp(plan.Ftype,'BSFFLT') ) 
                    if(~plan.XFFTdir)
                        error('You have to set a library directory in the property libdir')
                        else plan=precomputations(plan);
                    end%if
                end%if
               
            case 'BSFFLT'
                %check if a library directory is set
                if(strcmp(plan.Ftype,'NFFLT') || strcmp(plan.Ftype,'BSFFLT') ) 
                    if(~plan.XFFTdir)
                        error('You have to set a library directory in the property libdir')
                        else plan=precomputations(plan);
                    end%if
                end%if
            
           
            otherwise
                plan=precomputations(plan);
        end






      
        
    end%constructor
end%methods
end%class
    


