% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $
%
function f=mtimes_XFFLT_smallblocks(plan,fhat)
    
    M2=size(fhat,1);
    M1=size(plan.MY,1);
    f=zeros(M1,1);
    

    %case y in Y_M
    if(plan.Y(end,1)==plan.M) 
        tmp=plan.fhandle(plan,fhat); %XFFT      
        f(plan.Y(end,2):plan.Y(end,3))=tmp(plan.Y(end,2):plan.Y(end,3)); 
        
    end%if    
    
	vec_K=plan.Omega(:,1); %vector of indices of nonempty boxes Omega_K

    Fhat=zeros(M2,plan.q); %Fhat as repmat just for the boxes Omega_l of approximation case

    for tmpK=1:plan.Omega_num
        
        K=vec_K(tmpK);               
        c_Omega_K=3*plan.maxxi/2^(K+1);
        w_Omega_K=plan.maxxi/2^K;
        xi_cheb=c_Omega_K+1/2*w_Omega_K*plan.t;
               
        Omega_K=plan.MOmega(plan.Omega(tmpK,2):plan.Omega(tmpK,3)); %row vector of nodes in Omega_K
        
        tmpfhat=fhat(plan.Omega(tmpK,2):plan.Omega(tmpK,3));
        num=size(Omega_K,1);
        
        %Lagrange matrix LOmega
        L=zeros(num,plan.q);
        for r=1:plan.q
            L(:,r)=LagPol(plan.q,Omega_K,xi_cheb,r); %L_s^{Omega_k}(xi_0^\Omega_K,...,xi_{q-1}^\Omega_K)
        end
        Fhat(plan.Omega(tmpK,2):plan.Omega(tmpK,3),:)=repmat(tmpfhat,1,plan.q).*L; 
    end%for tmpK
    
  	for tmpI=1:plan.Iend
        tmpf=0;
        
        I=plan.Y(tmpI,1);
        Y_I=plan.MY(plan.Y(tmpI,2):plan.Y(tmpI,3)); %column vector of nodes in Y_I
        Ysize=size(Y_I,1);%number of nodes in box Y_I
   
        idxK=find((vec_K>=plan.l(tmpI)+1) & (vec_K<=plan.m(tmpI)-1)); %indexnumber in Omega with the condition in find
        h=zeros(M2,1);  
        idxKgm=find(vec_K>=plan.m(tmpI));

        
        %Approximation case l>m_I
        if(~isempty(idxKgm))
            h(1:(plan.Omega(idxKgm(1),3))) = fhat(1:plan.Omega(idxKgm(1),3));      
        end%if
        S=plan.fhandle(plan,h); %XFFT
        
        c=3*plan.maxy/2^(I+1);
        w=plan.maxy/2^I;
        y_cheb=c+1/2*w*plan.t;
        
        
        %Computation of h^{Y_I}  
        if(plan.m(tmpI) >1)
            %compute the pairs Y_I and Omega_K which have small blocks
            exactblock=zeros(size(idxK));
            exsum=zeros(Ysize,1);
            for tmpK=1:length(idxK)
                Omegasize=size(plan.MOmega(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3)),1);
                
                %exact computation for small blocks
                if (Ysize*Omegasize<=plan.blocksize) 
                    exactblock(tmpK)=1;
                    fprintf('exact evaluation for I=%d and K=%d\n',I,tmpK)
                    %do exact computation for f here  
                    X_I=plan.MX(plan.Y(tmpI,2):plan.Y(tmpI,3),1);
                    vecf_K=fhat(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3),:);
                    Omega_K=plan.MOmega(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3),1); %column vector of nodes in Omega_K
                    MZ=X_I./plan.N+1i*Y_I./(2*pi);
                    exsum=exsum+exp(2*pi*1i*MZ*Omega_K.')*vecf_K;
                else
                    fprintf('approximation for I=%d and K=%d later\n',I,tmpK)                   
                end%if          
            end%tmpK
            
            %add exact part
            f(plan.Y(tmpI,2):plan.Y(tmpI,3))=f(plan.Y(tmpI,2):plan.Y(tmpI,3))+exsum;
            
            idxK_approx=idxK(exactblock==0); % vector of indizes of vec_K, where we have to approximate
            idxK=idxK_approx;%update index vector
            
            %do approximation 
            if ( ~isempty(idxK) )
            
                                                       
                sumK=vec_K(idxK); %number K of non empty boxes K
                Fhatnew=zeros(M2,plan.q);
                for tmp=1:length(sumK)
                    K=sumK(tmp);

                    c_Omega_K=3*plan.maxxi/2^(K+1);
                    w_Omega_K=plan.maxxi/2^K;
                    xi_cheb=c_Omega_K+1/2*w_Omega_K*plan.t;
                    E=exp(-y_cheb.'*xi_cheb); %kernel matrix

                    %Omega_K=plan.MOmega(plan.Omega(idxK(tmp),2):plan.Omega(idxK(tmp),3)); %row vector of nodes in Omega_K
                   tmpFhat=Fhat(plan.Omega(idxK(tmp),2) : plan.Omega(idxK(tmp),3),:);
                    tmpFhat=tmpFhat*(E.');


                    Fhatnew(plan.Omega(idxK(tmp),2):plan.Omega(idxK(tmp),3),:)=tmpFhat;

                end%for tmp    

                F=plan.fhandle(plan,Fhatnew); 
                F=F(plan.Y(tmpI,2):plan.Y(tmpI,3),:);

                %Lagrange matrix LY
                LYI=zeros(size(Y_I,1),plan.q);
                for r=1:plan.q
                    LYI(:,r)=LagPol(plan.q,Y_I,y_cheb,r); %L_s^{Y_I}(y_0^Y_I,...,y_{q-1}^Y_I)
                end

                tmpf=sum(LYI.*F,2);
            end%if, do approximation
            
        end%if 
    
        f(plan.Y(tmpI,2):plan.Y(tmpI,3))=f(plan.Y(tmpI,2):plan.Y(tmpI,3)) + tmpf + S(plan.Y(tmpI,2):plan.Y(tmpI,3)); 
            
	end%for tmpI
      
  
    
end%mtimes