% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $
%
function f=mtimes_XFFLT_smallblocks(plan,fhat)

    n=size(fhat,2); %number of right hand sides
    M2=size(fhat,1);
    M1=size(plan.MY,1);
    f=zeros(M1,n);

    %Approximation case Y_I=Y_M
    if(plan.Y(end,1)==plan.M)    
        tmp=plan.fhandle(plan,fhat); %XFFT
        f(plan.Y(end,2):plan.Y(end,3),:)=tmp(plan.Y(end,2):plan.Y(end,3),:);     
    end    
     
    vec_K=plan.Omega(:,1); %vector of indices of nonempty boxes Omega_K

     
    %Computation for all boxes Y 
	for tmpI=1:plan.Iend
        
        I=plan.Y(tmpI,1);
        Y_I=plan.MY(plan.Y(tmpI,2):plan.Y(tmpI,3)); %column vector of nodes in Y_I
        Ysize=size(Y_I,1);%number of nodes in box Y_I
        
        idxK=find((vec_K>=plan.l(tmpI)+1) & (vec_K<=plan.m(tmpI)-1)); %indexnumber in Omega with the condition in find
        idxKgm=find(vec_K>=plan.m(tmpI)); %indexnumber in Omega with the condition in find

        
        %Approximation by 1 for the Laplace part
        h=zeros(M2,n);
        if(~isempty(idxKgm))
            h(1:(plan.Omega(idxKgm(1),3)),:) = fhat(1:plan.Omega(idxKgm(1),3),:);      
        end%if
        S=plan.fhandle(plan,h); %XFFT
        
        
        
        fhat_CupOmega=zeros(M2,n);       
        c=3*plan.maxy/2^(I+1);
        w=plan.maxy/2^I;
        y_cheb=c+1/2*w*plan.t;
        
        if(plan.m(tmpI) >1)
	
            %compute the pairs Y_I and Omega_K which have small blocks
            exactblock=zeros(size(idxK));
            exsum=zeros(Ysize,n);
            for tmpK=1:length(idxK)
                Omegasize=size(plan.MOmega(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3)),1);
                
                %exact computation for small blocks
                if (Ysize*Omegasize<=plan.blocksize) 
                    exactblock(tmpK)=1;
                    fprintf('exact evaluation for I=%d and K=%d\n',I,tmpK)
                    %do exact computation for f here  
                    X_I=plan.MX(plan.Y(tmpI,2):plan.Y(tmpI,3),1);
                    vecf_K=fhat(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3),:);
                    Omega_K=plan.MOmega(plan.Omega(idxK(tmpK),2):plan.Omega(idxK(tmpK),3),1); %column vector of nodes in Omega_K
                    MZ=X_I./plan.N+1i*Y_I./(2*pi);
                    exsum=exsum+exp(2*pi*1i*MZ*Omega_K.')*vecf_K;
                else
                    fprintf('do approximation for I=%d and K=%d later\n',I,tmpK)                   
                end%if          
            end%tmpK
            
            %add exact part
            f(plan.Y(tmpI,2):plan.Y(tmpI,3),:)=f(plan.Y(tmpI,2):plan.Y(tmpI,3),:)+exsum;
            
            idxK_approx=idxK(exactblock==0); % vector of indizes of vec_K, where we have to approximate
            idxK=idxK_approx;%update index vector

            %do approximation 
            if ( ~isempty(idxK) )
                    %set Fourier coefficients fhat_k : k\in Omega_K
                    %l_I+1<=K<=m_I-1 for K, where f was not computed exact
                    for tmpidxK=1:length(idxK)
                        fhat_CupOmega(plan.Omega(idxK(tmpidxK),2) : plan.Omega(idxK(tmpidxK),3),:)=fhat(plan.Omega(idxK(tmpidxK),2) : plan.Omega(idxK(tmpidxK),3),:); 
                    end
                   
                    sumK=vec_K(idxK); %number K of non empty boxes K
                    V=zeros(plan.q,n*Ysize);
                    G=zeros(M2,plan.q);
         
                    for tmp=1:length(sumK)
                        K=sumK(tmp);
                    
                        c_Omega_K=3*plan.maxxi/2^(K+1);
                        w_Omega_K=plan.maxxi/2^K;
                        xi_cheb=c_Omega_K+1/2*w_Omega_K*plan.t;
                        E=exp(-y_cheb.'*xi_cheb); 

                        Omega_K=plan.MOmega(plan.Omega(idxK(tmp),2):plan.Omega(idxK(tmp),3)); %row vector of nodes in Omega_K
                    
                        L=zeros(size(Omega_K,1),plan.q);
                        for r=1:plan.q
                            L(:,r)=LagPol(plan.q,Omega_K,xi_cheb,r); %L_s^{Omega_k}(xi_0^\Omega_K,...,xi_{q-1}^\Omega_K)
                        end                 
                        G(plan.Omega(idxK(tmp),2):plan.Omega(idxK(tmp),3),:) = (L*(E.'));
                    end%for tmp    
                    
                    for s=1:plan.q
                        g_s=fhat_CupOmega.*repmat(G(:,s),1,n); %g_s in M2 x n
                        tmpv=plan.fhandle(plan,g_s); %tmpv in M1 x n
                        tmpv=tmpv(plan.Y(tmpI,2):plan.Y(tmpI,3),:);
                        tmpv=tmpv(:);
                        V(s,:)=tmpv;
                        %for l=1:n
                        %    g_s=fhat_CupOmega(:,l).*G(:,s);
                        %    tmpv=plan.fhandle(plan,g_s); %N*ifft(g_s);
                        %    V(s,(l-1)*Ysize+1:l*Ysize)=tmpv(plan.Y(tmpI,2):plan.Y(tmpI,3));
                        %end %for l
                    end%s
                
                    LYI=zeros(size(Y_I,1),plan.q);
                    for r=1:plan.q
                        LYI(:,r)=LagPol(plan.q,Y_I,y_cheb,r); %L_s^{Y_I}(y_0^Y_I,...,y_{q-1}^Y_I)
                    end
                    %V
                    %(sum(LYI.*V))   
               
                    %pause
                    %f(plan.Y(tmpI,2):plan.Y(tmpI,3))=(sum(LYI.'.*V)).'+S(plan.Y(tmpI,2):plan.Y(tmpI,3)); 

                    for l=1:n
                        f(plan.Y(tmpI,2):plan.Y(tmpI,3),l)=f(plan.Y(tmpI,2):plan.Y(tmpI,3),l)+(sum(LYI.'.*V(:,(l-1)*Ysize+1:l*Ysize))).';
                    end%for l

            end%isempty idxk
            %Add approximation case by 1 of Laplace part
            f(plan.Y(tmpI,2):plan.Y(tmpI,3),:)=f(plan.Y(tmpI,2):plan.Y(tmpI,3),:)+S(plan.Y(tmpI,2):plan.Y(tmpI,3),:);
        end%if m.tmpI 
       
	end%for tmpI
      
    
	
  
    
end%mtimes