% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function u=mtimes(plan,f)
% computing the butterfly sparse Fourier transform
%
% u=mtimes(plan,f)
%
% INPUT
%   plan  object of class sparse_FFT1D
%   f     complex column vector with same length as plan.MOmega 
%
% OUTPUT
%   u  complex column vector with same length as plan.MX
	L=log2(plan.N);
	cA=plan.N/2;
	wA=plan.N;
	xA=cA+plan.alpha*wA; %grid in A
	mA=0;
	B_Boxes=plan.LastlevelB;
	
    switch(plan.option)
        case 'Mtb'
            wB=1;
    
            %computing coefficient vectors for l=0 , A=X_00
            F_cell=zeros(plan.p+2,size(B_Boxes,1));
            for j=1:size(B_Boxes,1) 
                %parameters of actuell box B
                tmp=B_Boxes(j,:);
                mB=tmp{1};
                B_index=tmp{2};
                cB=mB+1/2; 
                %computing the function values of the exact functions u^B
                u_AB=comp_first_func_val1D(plan.MOmega,plan.p,plan.N,xA,B_index,f);%function values
                %computing the coefficient vectors to the vector of function values
                f_AB=comp_first_coeff1D(plan.invG,plan.beta,plan.N,xA,cA,cB,wB,u_AB);
                tmp=[mA;mB;f_AB]; 
                F_cell(:,j)=tmp;
            end%for
	
            %Computing new function values from recursive from the previous level
            for l=1:L
                l
                numSonsLevel_l=plan.numSonsLevel(l);
                A_Param=plan.ABoxes{l}; 
                B_Param=plan.BBoxes{l}; 
                F_cell=comp_new_coeff1D(plan.invG,plan.H,plan.alpha,plan.beta,plan.p,plan.N,numSonsLevel_l,l,L,A_Param,B_Param,F_cell);
            end%for  
	 
            %last step: evaluation in the nodes MX
            cB=plan.N/2;
            wB=plan.N;
            M1=size(plan.MX,1);
            u=zeros(M1,1);
            A_Boxes=plan.LastlevelA; %boxes A of level L
            c2piiN=2*pi*1i/plan.N;
	
            xiB=cB+wB*plan.beta; %equispaced grid in B
            for r=1:size(F_cell,2)
                tmp=F_cell(:,r);
                f_AB=tmp(3:end);
            	tmpA=A_Boxes(r,:); %set A
                A_index=tmpA{2};
                vec_x=plan.MX(A_index);
                for s=1:plan.p
                    xi_s=xiB(s);
                    u(A_index)=u(A_index)+exp(c2piiN*vec_x*xi_s)*f_AB(s);
                end
            end%for r
        
        case {'Ltb','Ltb*'}
            
            %computing function values for l=0, A=X_00
            U_cell=zeros(plan.p+2,size(B_Boxes,1));
            for j=1:size(B_Boxes,1)
                %parameter of the current box B
                tmp=B_Boxes(j,:);
                mB=tmp{1};
                B_index=tmp{2};
                %computing the function values of the exact functions u^B
                u_AB=comp_first_func_val1D(plan.MOmega,plan.p,plan.N,xA,B_index,f);%function values
                tmp=[mA;mB;u_AB]; 
                U_cell(:,j)=tmp;
            end%for j

            %computing new function values from recursive from the previous level    
            for l=1:L
                l
                numSonsLevel_l=plan.numSonsLevel(l);
                A_Param=plan.ABoxes{l}; 
                B_Param=plan.BBoxes{l};
                U_cell=comp_new_func_val1D(plan.alpha,plan.p,plan.N,numSonsLevel_l,l,L,A_Param,B_Param,U_cell,plan.Lleft,plan.Lright);
            end%for
 
            %last step: evaluation in the nodes MX
            cB=plan.N/2;
        	wB=plan.N;
            wA=1;
            M1=size(plan.MX,1);
            u=zeros(M1,1);
            A_Boxes=plan.LastlevelA; %A-Boxes of level L
        	c2piiN=2*pi*1i/plan.N;
            c2piiNpwB=2*pi*1i*wB/((plan.p-1)*plan.N);

            switch(plan.option)
                case 'Ltb*'
                    for r=1:size(U_cell,2)
                        tmp=U_cell(:,r);
                        mA=tmp(1);
                        u_AB=tmp(3:end);
    
                        tmpA=A_Boxes(r,:);
                        A_index=tmpA{2};
		
                        cA=wA*(mA+1/2);
                        xA=cA+plan.alpha.*wA;	     
				
        				vec_x=plan.MX(A_index);
                		e=exp(c2piiN*vec_x*(cB+wB/2));
                        d=exp(-c2piiN*xA*(cB+wB/2)).';
            
                        lag=plan.Laglast(A_index,:);
                        u(A_index)=(e.*(lag*(d.*u_AB)));
                    end%for r
  
                case 'Ltb'
                    denom=zeros(plan.p,1); %denominator of lagrange polynomial
                    zalpha=exp(-2*pi*1i*plan.alpha/(plan.p-1));
                    for s=1:plan.p
                        denom(s)=1;
                        for j=[1:s-1,s+1:plan.p]
                            denom(s)=denom(s)*(zalpha(s)-zalpha(j));
                        end%for j
                    end %for s
			
                    unitp=ones(plan.p-1,1);
                    for r=1:size(U_cell,2)
                        tmp=U_cell(:,r);
                        mA=tmp(1);
                        u_AB=tmp(3:end);
				
                        tmpA=A_Boxes(r,:);
                        A_index=tmpA{2};
		
                        cA=wA*(mA+1/2);
                        xA=cA+plan.alpha.*wA;	     
				
                        vec_x=plan.MX(A_index);
                        e=exp(c2piiN*vec_x*(cB+wB/2));
                        d=exp(-c2piiN*xA*(cB+wB/2)).';
                				
                        numA=length(A_index);               
                        lag=zeros(numA,plan.p);
                        vec_z=exp(-c2piiNpwB*(vec_x-cA));
				
                        %tmpz=repmat(vec_z.',plan.p-1,1); %slow
                        tmpz=unitp*(vec_z.');
                        tmpz=tmpz(:);
                        unitA=ones(1,numA);
                        for s=1:plan.p
                            tmpzalpha=zalpha([1:s-1,s+1:plan.p]);	
                            %tmp=repmat(tmpzalpha,1,numA); %slow
                            tmp=(tmpzalpha.')*unitA;			
                            tmp=tmp(:);
                            numer=tmpz-tmp;
                            numer=reshape(numer,plan.p-1,numA);
                            numer=prod(numer,1); %numerator of lagrange polynomial, row-vector
                            lag(:,s)=numer.'/denom(s);
                        end
                        u(A_index)=(e.*(lag*(d.*u_AB)));
                    end%for r	
		
            end%switch
    end%switch
  
end%mtimes
