% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [F]=comp_new_coeff1D(invG,H,alpha,beta,p,N,numSonsLevel_l,l,L,A_Param,B_Param,F_cell)
	%disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))  
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))
 
	c2piiN=2*pi*1i/N;
  
    %parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	mA=tmpA(1);
	mP=floor(mA/2);

	parent=0; %counter
	
    %initialisations:
	F=zeros(p+2,size(A_Param,1)*size(B_Param,1));
	
	wA=N/2^l;
	wB=N/2^(L-l);
	wS=N/2^(L-l+1); %S son of B
	
	for j=1:size(A_Param,1)
		
		%actuell box A
		tmpA=A_Param(j,:);
		mA=tmpA(1);
		cA=wA*(mA+1/2);
		xA=cA+alpha.*wA; %grid in A
    
		%test, if the parent box changes
		if (  (j~=1) && ( mP~=floor(mA/2)   ))
			parent=parent+1;
			mP=floor(mA/2);
			%disp('different parent box')
			%else disp('same parentbox')
		end
        
		col=parent*numSonsLevel_l+1;
    		
		e2_AS=(exp(c2piiN*cA*wS*beta)).'; %column vector of diagonal matrix E_2^{AS}
        invd2_AB=(exp(-c2piiN*cA*wB*beta)).'; %column vector of diagonal matrix (D_2^{AB})^{-1}
		
    
		%compute f^{AB} from f^{PS}
		for k=1:size(B_Param,1)
			tmpB=B_Param(k,:);
			mB=tmpB(1);
			cB=wB*(mB+1/2);
			numSonsB=tmpB(2); %number of sons of B
			u_AB=zeros(p,1);
      
			for r=0:numSonsB-1
                tmpf=F_cell(3:end,col+r); %select f^(P S)
                %box parameters of the son
                mS=F_cell(2,col+r); 
                cS=wS*(mS+1/2); 
				c_AS=(exp(c2piiN*xA.*(cS-cB))).'; %column vector of diagonal matrix C^{AS}
                u_AB=u_AB+c_AS.*(H*(e2_AS.*tmpf));
			end%for
	
			col=col+numSonsB;
            f_AB=invd2_AB.*(invG*u_AB);
      
			F(:,(j-1)*size(B_Param,1)+k)=[mA;mB;f_AB];
      
		end%for
	end%for
end%function