% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [F]=comp_new_coeff2D(invG,H,alpha,beta,p,N,numSonsLevel_l,l,L,A_Param,B_Param,F_cell)
    %disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))
 
	c2piiN=2*pi*1i/N;
  
    %parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	m1A=tmpA(1);
	m2A=tmpA(2);
	m1P=floor(m1A/2);
	m2P=floor(m2A/2);

  	parent=0;%counter
	
    %initialisations:
	wA=N/2^l;
	wB=N/2^(L-l);
    wS=N/2^(L-l+1);
    F=zeros(p^2+4,size(A_Param,1)*size(B_Param,1));
    
	for j=1:size(A_Param,1)

		%actuell box A
		tmpA=A_Param(j,:);
		m1A=tmpA(1);
		m2A=tmpA(2);
		cA=wA*[m1A+1/2,m2A+1/2];
		xA1=cA(1)+alpha.*wA; %grid in A_1
		xA2=cA(2)+alpha.*wA; %grid in A_2
	
		%test, if the parent box changes
		if (  (j~=1) && ( m1P~=floor(m1A/2)  || (m2P~=floor(m2A/2) ) ))
			parent=parent+1;
			m1P=floor( m1A /2 );
			m2P=floor( m2A /2 );
			%disp('different parent box')
			%else disp('same parentbox')
		end	
		col=parent*numSonsLevel_l+1;
		
		e2_A1S1=(exp(c2piiN*cA(1)*wS*beta)).'; %column vector of diagonal matrix E_2^{A_1 S_1}
		e2_A2S2=(exp(c2piiN*cA(2)*wS*beta)).'; %column vector of diagonal matrix E_2^{A_2 S_2 }
		invd2_A1B1=(exp(-c2piiN*cA(1)*wB*beta)).'; %column vector of diagonal matrix (D_2^{A_1 B_1})^{-1}
		invd2_A2B2=(exp(-c2piiN*cA(2)*wB*beta)).'; %column vector of diagonal matrix (D_2^{A_2 B_2})^{-1}
        

		%compute f^{AB} from f^{PS}   
		for k=1:size(B_Param,1)
			%box parameters of B
			tmpB=B_Param(k,:);
			m1B=tmpB(1);
			m2B=tmpB(2);
			cB=wB*[m1B+1/2,m2B+1/2];
        
			numSonsB=tmpB(3); %number of sons of B

	        u_AB=zeros(p^2,1);

			for r=0:numSonsB-1
								
				%box parameters of the son
				m1S=F_cell(3,col+r); 
				m2S=F_cell(4,col+r);
				cS=wS*[m1S+1/2,m2S+1/2];
              
				tmpf=F_cell(5:end,col+r);  %select f^(P S)
				tmpF=reshape(tmpf,p,p);
                				
				c_A1S1=( exp(c2piiN*xA1.*(cS(1)-cB(1))) ).'; %column vector of diagonal matrix C^{A_1 S_1}
				c_A2S2=( exp(c2piiN*xA2.*(cS(2)-cB(2))) ).'; %column vector of diagonal matrix C^{A_2 S_2} 
				
				tmpU=(c_A1S1*c_A2S2.').*(H*((e2_A1S1*e2_A2S2.').*tmpF)*H.');
				u_AB=u_AB+tmpU(:);
			end%for

			col=col+numSonsB;
			
			tmpU_AB=reshape(u_AB,p,p);
			tmpF_AB=invG*tmpU_AB*invG.';
			tmpF_AB=(invd2_A1B1*invd2_A2B2.').*tmpF_AB;
		
		    tmp=[m1A;m2A;m1B;m2B;tmpF_AB(:)];
		    F(:,(j-1)*size(B_Param,1)+k)=tmp;
	end%for
end%for

end%function