% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [f_AB]=comp_first_coeff3D(invG,beta,p,N,xA1,xA2,xA3,cA,cB,wB,u_AB)
    
c2piiN=2*pi*1i/N;
%column vectors of the diagonal matrices (D_1/2/3^{A_1 B_1)^{-1},(D_1/2/3^{A_2 B_2)^{-1} 
invd1_A1B1=(exp(-c2piiN*xA1*cB(1))).';
invd2_A1B1=(exp(-c2piiN*cA(1)*wB*beta)).';
invd1_A2B2=(exp(-c2piiN*xA2*cB(2))).';
invd2_A2B2=(exp(-c2piiN*cA(2)*wB*beta)).';
invd1_A3B3=(exp(-c2piiN*xA3*cB(3))).';
invd2_A3B3=(exp(-c2piiN*cA(3)*wB*beta)).';


invD1_AB=zeros(p,p,p);
invD2_AB=zeros(p,p,p);
for t=1:p
	invD1_AB(:,:,t)=invd1_A3B3(t)*(invd1_A1B1*invd1_A2B2.');
	invD2_AB(:,:,t)=invd2_A3B3(t)*(invd2_A1B1*invd2_A2B2.');

end
%solving linear system of equals Mf^AB=u^AB by using tensor products
tensorU_AB=reshape(u_AB,p,p,p);
tensorF_AB=invD1_AB.*tensorU_AB;
tensorF_AB=nmode3D_coeff(tensorF_AB,invG,p);
tensorF_AB=invD2_AB.*tensorF_AB;
f_AB=tensorF_AB(:);

end