% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [f_AB]=comp_first_coeff4D(invG,beta,p,N,dA1,dA2,dA3,dA4,cA,cB,wB,u_AB)
%Input:
    %dA1,dA2,dA3,dA4 vectors for diagonal matrices
    % cA : center point of box A cA=[c1A;c2A];
    % wA : diameter of box A
    % cB : center point of box B cB=[c1B;c2B];
    % wB : diameter of box B
    % u_st: u_st^(AB)(x_st) column vector of length p^4
	
c2piiN=2*pi*1i/N;

%Spalten-Vektoren der Diagonalmatrizen M11^(-1),M12^(-1),M21^(-1),M22^(-1)
v_invM11=(exp(-c2piiN*dA1.*cB(1))).';
v_invM12=(exp(-c2piiN*cA(1)*wB*beta)).';
v_invM21=(exp(-c2piiN*dA2.*cB(2))).';
v_invM22=(exp(-c2piiN*cA(2)*wB*beta)).';
v_invM31=(exp(-c2piiN*dA3.*cB(3))).';
v_invM32=(exp(-c2piiN*cA(3)*wB*beta)).';
v_invM41=(exp(-c2piiN*dA4.*cB(4))).';
v_invM42=(exp(-c2piiN*cA(4)*wB*beta)).';

D_1=zeros(p,p,p,p);
D_2=zeros(p,p,p,p);
for t1=1:p
	for t2=1:p 
		D_1(:,:,t2,t1)=v_invM42(t1)*v_invM32(t2)*(v_invM12*v_invM22.');
		D_2(:,:,t2,t1)=v_invM41(t1)*v_invM31(t2)*(v_invM11*v_invM21.');
	end
end
tensorU_AB=reshape(u_AB,p,p,p,p);
tensorF_AB=D_2.*tensorU_AB;

tensorF_AB=nmode4D_coeff(tensorF_AB,invG,p);

tensorF_AB=D_1.*tensorF_AB;	

f_AB=tensorF_AB(:);
end