% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [U]=comp_new_func_val4D(alpha,p,N,numSonsLevel_l,l,L,A_Param,B_Param,U_cell,Lleft,Lright)

	%disp(sprintf('number of boxes A in nevel l= %d',size(A_Param,1)))  
	%disp(sprintf('number of Boxex B in level L-l=%d',size(B_Param,1)))
	
	c2piiN=2*pi*1i/N;
  
	%parameter of the box A and the parent box P of A
	tmpA=A_Param(1,:);
	m1A=tmpA(1);
	m2A=tmpA(2);
	m3A=tmpA(3);
	m4A=tmpA(4);
	mP1=floor(m1A/2);
	mP2=floor(m2A/2);
	mP3=floor(m3A/2);
	mP4=floor(m4A/2);
  
	parent=0; %counter
	
	%initialisations:
	U=zeros(p^4+8,size(A_Param,1)*size(B_Param,1));
  	wA=N/2^l;
	wP=N/2^(l-1);
	wS=N/2^(L-l+1);
  
	for j=1:size(A_Param,1)
		
		%actuell box A
		tmpA=A_Param(j,:);
		m1A=tmpA(1);
		m2A=tmpA(2);
		m3A=tmpA(3);
		m4A=tmpA(4);
		cA=wA*[m1A+1/2,m2A+1/2,m3A+1/2,m4A+1/2];
		xA1=cA(1)+alpha*wA; %Chebyshev grid in A_1
		xA2=cA(2)+alpha*wA; %Chebyshev grid in A_2
		xA3=cA(3)+alpha*wA; %Chebyshev grid in A_3
		xA4=cA(4)+alpha*wA; %Chebyshev grid in A_4
	
		%test, if the parent box changes
		if (  (j~=1) && (  ( (mP1~=floor(m1A/2) )  || (mP2~=floor(m2A/2)) ) ||  ((mP3~=floor(m3A/2)) || (mP4 ~=floor(m4A/2))  ) ) ) 
			parent=parent+1;
			mP1=floor(m1A/2);
			mP2=floor(m2A/2);
			mP3=floor(m3A/2);
			mP4=floor(m4A/2);
			%disp('different parent box')
		    %else disp('same parentbox')
		end
		col=parent*numSonsLevel_l+1;
		
		%grid in the parent box P of A   
		cP=wP*[mP1+1/2,mP2+1/2,mP3+1/2,mP4+1/2];
		xP1=cP(1)+alpha*wP;
		xP2=cP(2)+alpha*wP;
		xP3=cP(3)+alpha*wP;
		xP4=cP(4)+alpha*wP;
		
		%Lagrange matrices for A_1,A_2,A_3,A_4
		Lag={zeros(p,p),zeros(p,p),zeros(p,p),zeros(p,p)};
        mA=[m1A,m2A,m3A,m4A];
		for d=1:4
            %test, if A is the left or the right son
            if (mod(mA(d),2)==0)
                Lag{d}=Lleft;
            else
                Lag{d}=Lright;
			end
		end%for d
		
		%compute u^{AB} from u^{PS}
		for k=1:size(B_Param,1)
			%actuell box B
			tmpB=B_Param(k,:);
			m1B=tmpB(1);
			m2B=tmpB(2);
			m3B=tmpB(3);
			m4B=tmpB(4);
			numSonsB=tmpB(5);

			U_AB=zeros(p,p,p,p);

			for r=0:numSonsB-1
	            u_PS=U_cell(9:end,col+r); %select u^{P S}
				U_PS=reshape(u_PS,p,p,p,p);
			
				%Parameter der Sohn-Box
				m1S=U_cell(5,col+r); 
				m2S=U_cell(6,col+r);
				m3S=U_cell(7,col+r);
				m4S=U_cell(8,col+r);
				cS=wS*[m1S+1/2,m2S+1/2,m3S+1/2,m4S+1/2];
			  
				%column vectors of the diagonal matrices
				%S^{P_1S_1},S^{P_2S_2},S^{P_3 S_3},S^{P_4 S_4}
				s_P1S1=exp(-c2piiN*xP1*(cS(1)+wS/2)).';
				s_P2S2=exp(-c2piiN*xP2*(cS(2)+wS/2)).';
				s_P3S3=exp(-c2piiN*xP3*(cS(3)+wS/2)).';
				s_P4S4=exp(-c2piiN*xP4*(cS(4)+wS/2)).';
								
				%column vektors of the shift-matrix R^{A_1S_1}, R^{A_2S_2},
				%R^{A_3 S_3} R^{A_4 S_4}
				r_A1S1=exp(c2piiN*xA1*(cS(1)+wS/2)).';
				r_A2S2=exp(c2piiN*xA2*(cS(2)+wS/2)).';
				r_A3S3=exp(c2piiN*xA3*(cS(3)+wS/2)).';
				r_A4S4=exp(c2piiN*xA4*(cS(4)+wS/2)).';
				
				%tensors
				R_AS=zeros(p,p,p,p);
				S_PS=zeros(p,p,p,p);
				
				for s1=1:p
					for s2=1:p 
						R_AS(:,:,s2,s1)=r_A4S4(s1)*r_A3S3(s2)*(r_A1S1*r_A2S2.');
						S_PS(:,:,s2,s1)=s_P4S4(s1)*s_P3S3(s2)*(s_P1S1*s_P2S2.');
					end
				end
				
				U_PS=S_PS.*U_PS;
					
				tmpU=nmode4D(U_PS,Lag{1},Lag{2},Lag{3},Lag{4},p);
				tmpU=R_AS.*tmpU;
				U_AB=U_AB+tmpU;

		end%for
		
       col=col+numSonsB;   

       U(:,(j-1)*size(B_Param,1)+k)=[m1A;m2A;m3A;m4A;m1B;m2B;m3B;m4B;U_AB(:)];

	end%for
end%for

end%function