% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

classdef tree1D
properties
    sons={} % maximal 2 sons
    l=[];   % level l
    m=[];   % m - position parameter of the box
    ind=[]; % indices of the nodes from the matrix [row vector]
 end%properties
    
methods  

function X=tree1D( M,ind_input, N, l_input, m_input)
  %constructor
    X.l=l_input;
    X.m=m_input;
    %X.ind=ind_input;
    
    if (2^X.l==N ||isempty(ind_input))
      X.ind=ind_input;
    else
      %X.lastlevel=0;
      
      %decompose nodes and create sons
      Index=decompose_index1D(M,ind_input, X.l, X.m, N);         
      X.sons={};
      %Index{1}
      for k=1:length(Index)
        if(~isempty(Index{k}))
           switch(k)
            case 1                  
              son=tree1D(M, Index{k}, N, X.l+1, X.m*2); %left son
            case 2
              son=tree1D(M, Index{k}, N, X.l+1, X.m*2+1); %right son
           end%switch
               
           X.sons=[X.sons {son}];
        end%if
      end%for
    end %if         
end %function tree1D

%functions:

function out=countSons(X,l_input,m_input)
%Output: number of sons of the Box X_l,m
    if(X.l==l_input)
      if(X.m==m_input)
          out=length(X.sons);
      else out=0; % box doesn't exist ->no sons
      end%if
    elseif(l_input>X.l)
       out=0; 
       for k=1:length(X.sons)
          out=out+X.sons{k}.countSons(l_input,m_input);
      end%for
    else
        out=0;
    end%if
end%function countSons
 
function out=countSonsInLevel(X,l_input)
%Output: number of all sons in level l = number of boxes in level l+1
     out=0;
     if(X.l==l_input-1)
          for k=1:length(X.sons)
             out=out+length(X.sons{k}.sons);
          end%for
     elseif(l_input==X.l)
         out=length(X.sons);
     elseif(l_input > (X.l+1))
         for k=1:length(X.sons)
             out=out+X.sons{k}.countSonsInLevel(l_input);
         end%for
     end%if       
 end%function countSonsInLevel
 
function levelBoxes=getBoxes(X,l_input)
%Output: all boxes of level l as cellaray with rows {m,indices,number of sons}
  if(X.l==l_input) 
      if(~isempty(X.ind))
	c=X.countSons(X.l,X.m);
        levelBoxes={X.m,X.ind,c};
      else
          tmp=X.get_index();
	  c=X.countSons(X.l,X.m);
          levelBoxes={X.m, tmp,c};
      end%if
  else
      levelBoxes={};
      for k=1:length(X.sons)
          tmp=X.sons{k}.getBoxes(l_input);
          levelBoxes=[levelBoxes;  tmp];
      end%for   
  end%if
end%function



function levelPara=getLevelParaA(X,l_input)
%Output: all position parameters m of boxes in level l as column vector
  if(X.l==l_input) 
      levelPara=[X.m];
  else
      levelPara=[];
      for k=1:length(X.sons)
          tmp=X.sons{k}.getLevelParaA(l_input);
          levelPara=[levelPara; tmp];
      end%for
  end%if
end%function getLevelParaA

function levelPara=getLevelParaB(X,l_input)
 %Output: all position parameters m and number of sons of the box X_l,m of boxes in
%level l as 2-column matrix [m,c]

  if(X.l==l_input) 
      c=length(X.sons);
      levelPara=[X.m,c];
  else
      levelPara=[];
      for k=1:length(X.sons)
          tmp=X.sons{k}.getLevelParaB(l_input);
          levelPara=[levelPara; tmp];
      end
  end%if
end%function getLevelParaB 
  
  
  
%%%%% only functions for testing tree1D: %%%%%
function index_out=IndexInBox(X, l_input, m_input)
 %Output: indices from the node matrix, which lie in X_l,m
    if (X.l==l_input)
       if (X.m==m_input)
          index_out=X.get_index();
       else index_out=[]; %if box doesn't exist
       end%if
       
    else
       index_out=[];
       for k=1:length(X.sons)
           tmp=X.sons{k}.IndexInBox(l_input, m_input);
           index_out=[index_out tmp];
       end%for
    end%if
end%function IndexInBox
  
  
function index_out=get_index(X)
 %Output: all indices wich lie in the boxes of the tree X
    %if (X.lastlevel==1) 
	if(~isempty(X.ind))
	index_out=X.ind;
    else
      index_out=[];                                        
      for k=1:length(X.sons)
          tmp=X.sons{k}.get_index();                                      
          index_out=[index_out tmp];
      end %for                                              
	end%if
end%function index_out

end %methods
end %classdef