% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate$
% $Author$
% $Revision$
%
%This script computes the time of the Fast Laplace Transformation for dimension
%d=1 with respect to the problem sizes M1,M2.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show1D_times_LT


q=4;
%q=8;

%%
%add XFFT library
addpath('../..')

%add NFFT Library directories
%NFFTdir='/home/math/imelzer/software/nfft-3.3_optiplex/';
NFFTdir='/home/math/imelzer/software/nfft-3.3_Phi/'; 

%add utilities
addpath('../utilities');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ymax=(2*q-1)*log(2);

minL=1;
maxL=24;

maxLex=18;%naive computation only up to maxLex

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

d=1; %dimension

display(sprintf('test for q=%d',q))

vec_s=[];
t=[];
t_ex1=[];
t_ex2=[];
err=[];

    blocksize=16;
    
    L=minL;
    timetest=1;
while (L<=maxL && timetest <86400)
    
	display('=========================================================')
	display(L)
	N=2^L;
    vec_s=[vec_s,L];
    
    MY=ymax*sort(rand(N,1));
    fhat=complex(rand(N,1)-1/2,rand(N,1)-1/2);
    MOmega=(1:N).'; %equispaced Frequencies for all tests
    

    %MOmega=ximax*sort(rand(N,1));
    plan=XFLT('LT',MY,'rank',q,'MOmega',MOmega);
    %plan_block=XFLT('LT',MY,'rank',q,'MOmega',MOmega,'blocksize',blocksize);
            
    display('exact computation')
    if L<=maxLex
    	tic
            u_ex1=u_exactLT(MY,MOmega,fhat,1);
        t_ex1=[t_ex1;toc];
                
                
        tic
            u_ex2=u_exactLT(MY,MOmega,fhat,2);
        t_ex2=[t_ex2;toc];
    end%if
            
    savefile=sprintf('data/Phi_times1D_LT_q%d.mat',q);
    
                                 
    display('approximative computation')
	tic
		f=mtimes(plan,fhat);
		t=[t;toc];
        

        if L<=maxLex
            display('relative errors:')
            err1=norm(u_ex1-f,inf)/norm(fhat,1)
            err2=norm(u_ex2-f,inf)/norm(fhat,1)
		    err=[err;err1];
        end
        
        
        
    save(savefile,'q','vec_s','t_ex1','t_ex2','t','err1','err2','maxLex')
    
    L=L+1;
    timetest=t(end);
end