% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function [F]=approxF(MX,MOmega,p,N,cA,cB,wA,wB,alpha,beta)

%grids in A and B
X=cA+alpha*wA;
Xi=cB+beta*wB;
	
% 1. F1: u^B = F1*f
	F1=exp(2*pi*1i*X'*MOmega'/N);

% 2. invM = M^(-1) interpolation matrix of Mf^(AB)=u^B
	v_invM1=(exp(-2*pi*1i*cB/N*(cA + wA*alpha))).';
	v_invM2=(exp(-2*pi*1i*cA*wB/N*beta)).';
	G=zeros(p,p);
	for tmp=1:p
      G(:,tmp)=exp(2*pi*1i*(alpha).*(beta(tmp)));
	end
	invG=inv(G);
	invM=spdiags(v_invM2,0,p,p)*invG*spdiags(v_invM1,0,p,p);
	

% 3. F2: u^(AB) = F2*f^(AB)
	F2=exp(2*pi*1i*MX*Xi/N);
	
%Output:
	F=F2*invM*F1;


end%function