% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%This script computes the accuracy of the butterfly sparse FFT for dimension
%d=1  for fixed p with respect to L=log2 N.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show1D_relative_error_dependence_L.

%computes the accuracy for fixed p in dependence of L
%shows eps \approx C(L+1)
clear all;
addpath('../..');
addpath('../utilities')
addpath('data')

savefile='data/relative_error1D_dependence_L.mat';


minp=5;
maxp=9;
minL=5;
maxL=14;
vec_p=minp:maxp;
vec_L=minL:maxL;

err_const=inf*ones(maxp-minp+1,maxL-minL+1);

for p=1:maxp-minp+1
	
	for L=1:maxL-minL+1
		N=2^vec_L(L);
		M1=N;
		M2=N;
		%nodes in X
		MX=[N*rand(M1-1,1);0];
		%nodes in Omega
		MOmega=N*rand(M2,1);
		%constant coefficients
		f_const=ones(M2,1);
		%exact computation:
		u_ex_const=u_exact(MX,MOmega,f_const,N);
		
		
		%errors
		plan=sparse_FFT1D(MX,MOmega,vec_p(p),N,'Ltb');
		u_const=mtimes(plan,f_const);
		err_const(p,L)=norm((u_const-u_ex_const),inf)/norm(f_const,1);
		
		
		save(savefile,'vec_p','vec_L','M1','M2','err_const')
		
	end
end