% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%This script computes the relative error of the butterfly sparse FFT for dimension
%d=2 with respect to the local expansion degree p.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show2D_relative_error.

clear all;
addpath('../..');
addpath('../utilities')
addpath('data')
%parameters
d=2; %dimension
minp=2;	 %minimal number of apprroximation rank
maxp=16; %maximal number of approximation rank

L=10;  %figure 4.3. (c) 
%L=14; %figure 4.3. (d)

N=2^L;
savefile=sprintf('data/relative_error2D_L%d.mat',L);

%defining nodes
M1=N;
M2=N;

%nodes on Ellipsen in [0,N]x[0,N]
a=N/2;
b=N/5;
%matrix M_X:
	%t1=linspace(0,2*pi,M1)';
	t1=linspace(0,2*pi,M1-1)';
	x1=a+a*cos(t1);
	x2=a+b*sin(t1);
	%MX=[x1,x2];
	MX=[0,0;x1,x2];


%matrix M_Omega:
	t2=linspace(0,2*pi,M2)';
	xi1=a+b*sin(t2);
	xi2=a+a*cos(t2);
	MOmega=[xi1,xi2];

%coefficients
	f=complex(rand(M2,1)-1/2,rand(M2,1)-1/2); %random
	%f=ones(M2,1);%constant

display('exact computation')
	u_ex=u_exact(MX,MOmega,f,N);
	
display('approximative computation')
err_Ltb=[];
err_Mtb=[];
vec_p=[];
vec_kappa=[];

for p=minp:maxp
    display('=========================================================')
    display(p)
	
		display('Monomial-type basis')
		plan1=sparse_FFT2D(MX,MOmega,p,N,'Mtb');
		u_Mtb=mtimes(plan1,f);
		kappa=cond(plan1.invG);
        vec_kappa=[vec_kappa;kappa];
		clear plan1;
	
		display('Lagrange-type basis')
		plan2=sparse_FFT2D(MX,MOmega,p,N,'Ltb');
		u_Ltb=mtimes(plan2,f);
		clear plan2;
	
		tmp1=norm((u_Ltb-u_ex),inf)/norm(f,1);
		tmp2=norm((u_Mtb-u_ex),inf)/norm(f,1);
		err_Ltb=[err_Ltb;tmp1];
		err_Mtb=[err_Mtb;tmp2];
		vec_p=[vec_p;p];

	save(savefile,'vec_p','N','M1','M2','f','err_Ltb','err_Mtb','u_ex','vec_kappa','d')
end
