% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%This script computes the relative error of the butterfly sparse FFT for dimension
%d=3 with respect to the local expansion degree p.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show3D_relative_error.

clear all;
addpath('../..');
addpath('../utilities')
addpath('data')


%parameters:
d=3; %dimension
minp=2;	%minimal number of  number of apprroximation rank
maxp=15; %maximal number of  number of apprroximation rank

%L=4;
L=8;

N=2^L;

savefile=sprintf('data/relative_error3D_L%d.mat',L); 

%nodes on spheres
r1=N/2;
r2=2/5*N;

%matrix M_X:
	M1=N;
	X=randn(M1^2,3);
	nX=repmat(sqrt(sum(X.^2,2)),1,3);
	X=X./nX;
	MX=N/2+r1*X;
	MX=[MX;0,0,0]; %nodes spatial domain
	M1=size(MX,1);
	clear X nX;
%matrix M_Omega:
    M2=N;
	Omega=randn(M2^2,3);
	nOmega=repmat(sqrt(sum(Omega.^2,2)),1,3);
	Omega=Omega./nOmega;
	MOmega=N/2+r2*Omega; %nodes in frequency domain
	M2=size(MOmega,1);
	clear Omega nOmega;

%coefficients
f=complex(rand(M2,1)-1/2,rand(M2,1)-1/2);
%f=ones(M2^2,1);

u_ex=u_exact(MX,MOmega,f,N); %exact computation

display('approximative computation')
err_Ltb=[];
err_Mtb=[];
vec_p=[];
vec_kappa=[];

for p=minp:maxp
    display('=========================================================')
    display(p)
	
    display('Monomial-type basis')
	plan1=sparse_FFT3D(MX,MOmega,p,N,'Mtb');
	u_Mtb=mtimes(plan1,f);
	kappa=cond(plan1.invG);
	vec_kappa=[vec_kappa,kappa];
	clear plan1;
    
	display('Lagrange-type basis')
	plan2=sparse_FFT3D(MX,MOmega,p,N,'Ltb');
	u_Ltb=mtimes(plan2,f);
	clear plan2;
	
	tmp1=norm((u_Ltb-u_ex),inf)/norm(f,1);
	tmp2=norm((u_Mtb-u_ex),inf)/norm(f,1);
	err_Ltb=[err_Ltb;tmp1];
	err_Mtb=[err_Mtb;tmp2];
	vec_p=[vec_p;p];

	save(savefile,'vec_p','N','M1','M2','f','err_Ltb','err_Mtb','u_ex','vec_kappa','d')

	
end