% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%This script computes the relative error of the butterfly sparse FFT for dimension
%d=4 with respect to the local expansion degree p.
%The important computations will be saved in a mat file in the
%directory data, see the initialisation of savefile.
%For plotting you have to call the function show4D_relative_error.

clear all;
addpath('../..');
addpath('../utilities')
addpath('data')

%set parameters: 
d=4; %dimension
minp=2;	%minimal number of apprroximation rank
maxp=15; %maximal number of apprroximation rank

L=3;
%L=5;
savefile=sprintf('data/relativeError4D_L%d.mat',L); 


N=2^L;
%Knodes on 3-dimensional hyperplanes
r1=N/2;
r2=2/5*N;
%matrix M_X
	M1=N;
	X=randn(M1^3,4);
	nX=repmat(sqrt(sum(X.^2,2)),1,4);
	X=X./nX;
	MX=N/2+r1*X;
	MX=[0,0,0,0;MX];
	MX=unique(MX,'rows');
	M1=size(MX,1);
	clear X nX;
%matrix M_Omega
	M2=N;
	Omega=randn(M2^3,4);
	nOmega=repmat(sqrt(sum(Omega.^2,2)),1,4);
	Omega=Omega./nOmega;
	MOmega=N/2+r2*Omega;
	MOmega=unique(MOmega,'rows');
	M2=size(MOmega,1);
	clear Omega nOmega;


%coefficients
f=ones(M2^2,1); 

display('exact computation')
	u_ex=u_exact(MX,MOmega,f,N);

display('approximative computation')
err_Ltb=[];
err_Mtb=[];
vec_p=[];
vec_kappa=[];

for p=minp:maxp
    display('=========================================================')
    display(p)
	
		display('Monomial-type basis')
		plan1=sparse_FFT4D(MX,MOmega,p,N,'Mtb');
		u_Mtb=mtimes(plan1,f);
		kappa=cond(plan1.invG);
        vec_kappa=[vec_kappa;kappa];
		clear plan1;
	
		display('Lagrange-type basis')
		plan2=sparse_FFT4D_Ltb(MX,MOmega,p,N,'Ltb');
		u_Ltb=mtimes(plan2,f);
		clear plan2;
	
		tmp1=norm((u_Ltb-u_ex),inf)/norm(f,1);
		tmp2=norm((u_Mtb-u_ex),inf)/norm(f,1);
		err_Ltb=[err_Ltb;tmp1];
		err_Mtb=[err_Mtb;tmp2];
		vec_p=[vec_p;p];

	save(savefile,'vec_p','N','M1','M2','f','err_Ltb','err_Mtb','u_ex','vec_kappa','d')
end
