% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%Before you call this script, you have to call the script compute3D_times.
%Don't forget to set the file correct!

clear all;
addpath('data')
addpath('../utilities')

%evaluation of running times:
%file='times3D_p4_r900';  %figure 4.6. (c)
file='times3D_p8_r900'; %figure 4.6. (d)

load(file)
addpath('..')

L_begin=vec_L(1);
L_end=vec_L(end);

vec_N=2.^(vec_L);

N_begin=2;
N_end=length(vec_N);

vec_N2=vec_N(N_begin:end);
vec_L2=vec_L(N_begin:end);

C_ex=least_squares_4(vec_N(2:length(t_ex)),t_ex(2:length(t_ex)),d); %C*N^(2d-2)
C_Mtb=least_squares_5(vec_N2,t_Mtb(N_begin:N_end),d); %C*N^(d-1)logN
C_Ltb=least_squares_5(vec_N2,t_Ltb(N_begin:N_end),d); %C*N^(d-1)logN
C_Ltbstar=least_squares_5(vec_N2,t_Ltbstar(N_begin:N_end),d); %C*N^(d-1)logN

%larger figur
h = figure(1);
clf(h);
s = get(h,'Position');
set(h,'Position',[s(1:2),s(3)+200,s(4)+200]);

%%colored 
 h(1)=loglog(vec_N(1:length(t_ex)),t_ex,'bd');
 hold on
 h(2)=loglog(vec_N,t_Mtb,'rs');
 h(3)=loglog(vec_N,t_Ltb,'o');
 h(4)=loglog(vec_N,t_Ltbstar,'x');
 h(5)=loglog(vec_N2,C_Mtb*vec_N2.^(d-1).*vec_L2,'r-');
 h(6)=loglog(vec_N2,C_Ltb*vec_N2.^(d-1).*vec_L2,'-');
 h(7)=loglog(vec_N2,C_Ltbstar*vec_N2.^(d-1).*vec_L2,'-');
 h(8)=loglog(vec_N(1:8),C_ex*vec_N(1:8).^(2*d-2),'b-');
 h(9)=loglog(vec_N(8:end),C_ex*vec_N(8:end).^(2*d-2),'b:');
 
 set(h(6),'Color',[1,3/4,0]);%yellow
 set(h(3),'Color',[1,3/4,0]);%yellow
 set(h(4),'Color',[0,1/2,0]);%green
 set(h(7),'Color',[0,1/2,0]);%green
 gerade1=sprintf('naive, %.2e N^2',C_ex);
 gerade2=sprintf('Mtb, %.2e N^2 log_2 N',C_Mtb);
 gerade3=sprintf('Ltb, %.2e N^2 log_2 N',C_Ltb);
 gerade4=sprintf('Ltb*, %.2e N^2 log_2 N',C_Ltbstar);
 legend(gerade1,gerade2,gerade3,gerade4,'Location','NorthWest')
 
 
 
% %colored version for butterfly paper
% h(1)=loglog(vec_N,t_Mtb,'d-');
% hold on
% h(2)=loglog(vec_N,t_Ltb,'b>-');
% h(3)=loglog(vec_N,t_Ltbstar,'bx-');
% h(4)=loglog(vec_N(1:length(t_ex)),t_ex,'--');
% h(5)=loglog(vec_N(8:end),C_ex*vec_N(8:end).^(2*d-2),'-.');
% set(h(1),'Color',[0,1/2,0]);%green
% set(h(4),'Color',[1/2,0,1/2]);%magenta
% set(h(5),'Color',[1/2,0,1/2]);%magenta
% legend('Sect. 2.3.2','Sect. 2.3.2 *','Sect. 2.3.1','naive','Location','NorthWest')
% ylabel('time in s')
% xlabel('N')
% set(gca,'XTick',2.^(1:1:10))
% set(gca,'YTick',10.^(-2:1:8))

sprintf('naive, %.2e N^4',C_ex)
sprintf('Mtb, %.2e N^2 log_2 N',C_Mtb)
sprintf('Ltb, %.2e N^2 log_2 N',C_Ltb)
sprintf('Ltb*, %.2e N^2 log_2 N',C_Ltbstar)


axis([2,2^(10),10^(-2),10^6])

fs=14;
set(gca,'Fontsize',fs);
set(get(gca,'title'),'Fontsize',fs);
set(get(gca,'XLabel'),'Fontsize',fs);
set(get(gca,'YLabel'),'Fontsize',fs);

 for j=1:length(h)
 	set(h(j),'linewidth',2,'MarkerSize',8)
 end

hold off
savepic=strcat('pics/',file);
saveas(gcf,savepic,'png')