% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

%Before you call this script, you have to call the script compute4D_relative_error.
%Don't forget to set the file correct!

clear all;
addpath('data')
addpath('../utilities')

%sample files:
%constant coefficients
    %file='relative_error4D_constant_L3_hyperebene_r900';
    %file='relative_error4D_constant_L5_hyperebene_r900';
    
    file='relative_error4D_constant_L3';
    %file='relative_error4D_constant_L5';
    
load(file)

minp=vec_p(1);
maxp=vec_p(end);

%theoretical error estimate
err=error_estimate(d,N,vec_p);

%least squares fit
start1=1;
stop1=12;
vec_p1=vec_p(start1:stop1);
[k_Ltb,C_Ltb]=least_squares(vec_p1,err_Ltb(start1:stop1));

start2=1;
stop2=7;
vec_p2=vec_p(start2:stop2);
[k_Mtb,C_Mtb]=least_squares(vec_p2,err_Mtb(start2:stop2));

%least squares lines:
fprintf('Lagrange-type method: %.2f* %.2f^{-p} \n',C_Ltb,k_Ltb)
fprintf('Monomial-type method: %.2f* %.2f^{-p} \n',C_Mtb,k_Mtb)


%larger figure
h = figure(1);
clf(h);
s = get(h,'Position');
set(h,'Position',[s(1:2),s(3)+200,s(4)+200]);

%colored
h(1)=semilogy(vec_p,err_Ltb,'bd');
hold on;
h(2)=semilogy(vec_p,err_Mtb,'x');
h(3)=semilogy(vec_p1,C_Ltb*k_Ltb.^(-vec_p1),'b-');
h(4)=semilogy(vec_p2,C_Mtb*k_Mtb.^(-vec_p2),'-');
h(5)=semilogy(vec_p,err,'r--'); %theoretical estimate
set(h(2),'Color',[0,1/2,0]);%green
set(h(4),'Color',[0,1/2,0]);%green 
 
errfunc='error estimate';
set(get(get(h(3),'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
set(get(get(h(4),'Annotation'),'LegendInformation'),'IconDisplayStyle','off');
legend('Lagrange-type basis','Monomial-type basis',errfunc,'Location','SouthWest')
 
xlabel('p');
ylabel('\epsilon_2');
axis([2.95,maxp,10^(-16),10^0])
set(gca,'XTick',3:1:maxp)
vec_y=fliplr(0:1:16);
set(gca,'YTick',10.^(-vec_y))

fs=14;
 set(gca,'Fontsize',fs);
 set(get(gca,'title'),'Fontsize',fs);
 set(get(gca,'XLabel'),'Fontsize',fs);
 set(get(gca,'YLabel'),'Fontsize',fs);
 for j=1:length(h)
 	set(h(j),'linewidth',2,'MarkerSize',8);
 end
hold off

savepic=strcat('pics/',file);
saveas(gcf,savepic,'png')