% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 13:41:37 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6165 $

function u=u_exact(MX,MOmega,f,N)
% computes the exact sums for each dimension d, just for comparison with the approximative sparse FFT
%
% u=u_exact(MX,MOmega,f,N)
%
% INPUT
%    N       - domain parameter, must be a power of 2
%    MX      - sampling nodes in X=[0,N]^d as M1xd matrix
%    MOmega  - sampling nodes in Omega=[0,N]^d as M2xd matrix
%
% OUTPUT
%   u  complex column vector of length size(MX,1)

M1=size(MX,1);
M2=size(MOmega,1);
u=zeros(M1,1);

for k=1:M1
	for l=1:M2
			u(k)=u(k)+exp(2*pi*1i*MX(k,:)*MOmega(l,:).'/N)*f(l);
    end
end

end