% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate$
% $Author$
% $Revision$
%
function u=u_exactall(MY,MX,MOmega,N,fhat,varargin)

%
% u=u_exactall(MX,MOmega,f,N)
%
% INPUT
%    MX      - sampling nodes in X=[0,N] as row vector
%    MY      - sampling nodes in Y in R_+ as row vector
%    MOmega  - sampling nodes in Omega=[0,N] as row vector
%
% OUTPUT
%   u  complex column vector of length size(MX,1)

M1=size(MY,1);
M2=size(MOmega,1);
u=zeros(M1,1);


Z=MX./N+1i*MY./(2*pi);

if (~isempty(varargin))
   option=varargin{1};
else
    option=1;
end

switch(option)
    case 1
        for k=1:M2
 			u=u+(exp(2*pi*1i*Z*MOmega(k))*fhat(k));
         end
    case 2

        for j=1:M1
            for k=1:M2
                u(j)=u(j)+exp(2*pi*1i*Z(j)*MOmega(k))*fhat(k);
            end
        end
    otherwise
                display('unknown option for exact computation');

end

end