% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 14:16:54 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6169 $

%START2D test script for d=2
clear all;
addpath('numerical_tests/utilities')
%test for the routines

%setting parameters:
L=7;
p=8;
N=2^L;
stepA=N-1;
stepB=N-1;

%nodes on ellipses in [0,N]x[0,N]
%matrix X
t1=linspace(0,2*pi,stepA)';
a1=1/2*N;
b1=2/5*N;
x1=N/2+a1*cos(t1);
y1=N/2+b1*sin(t1);
MX=[x1,y1];
MX=[0,0;MX];
%MX=unique(MX,'rows');
M1=size(MX,1);

%matrix MOmega
t2=linspace(0,2*pi,stepB)';
a2=1/2*N;
b2=2/5*N;
x2=N/2+b2*sin(t2);
y2=N/2+a2*cos(t2);
MOmega=[x2,y2];
%MOmega=unique(MOmega,'rows');
M2=size(MOmega,1);

%coefficients
f=ones(M2,1);

%plans for different variants

%1. 
disp('Mtb: computation via monomial-type basis')
plan1=sparse_FFT2D(MX,MOmega,p,N,'Mtb'); %monomial type basis
     start1=tic;
 u1=mtimes(plan1,f);
     t1=toc(start1);
 %clear plan1;
 	 
%2. 
disp('Ltb: computation via Lagrange-type basis without precomputation of the polynomials in the last step') 
plan2=sparse_FFT2D(MX,MOmega,p,N,'Ltb'); 
	start2=tic;
u2=mtimes(plan2,f);
	t2=toc(start2);
clear plan2;

%3. 
disp('Ltb*: computation via Lagrange type basis with precomputation of the polynomials in the last step')
plan3=sparse_FFT2D(MX,MOmega,p,N,'Ltb*');
    start3=tic;
u3=mtimes(plan3,f);
	t3=toc(start3);
clear plan3;

%4. 
display('exact computation')
tic
    u_ex=u_exact(MX,MOmega,f,N);
t_ex=toc;


fprintf('\n')
disp(sprintf('test with parameters:'))
disp(sprintf('  N= %d',N))
disp(sprintf('  M1= %d',M1))
disp(sprintf('  M2= %d',M2))
disp(sprintf('  p= %d',p))
disp(sprintf('  time of the exact computation = %d', t_ex))

fprintf('\n')
%time
disp(sprintf('comparison of the three variants:'))
disp(sprintf('  time of Mtb  = %d',t1))
disp(sprintf('  time of Ltb  = %d',t2))
disp(sprintf('  time of Ltb* = %d',t3))
disp('-----------------------------------------------');

%absolute error
disp(sprintf('  Mtb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u1))))
disp(sprintf('  Ltb:  max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u2))))
disp(sprintf('  Ltb*: max_i |u_{ex}(i) - u_{approx}(i)|= %d',max(abs(u_ex-u3))))

%relative error
disp('----------------------------------------------');
disp(sprintf('  Mtb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u1-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb:  ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u2-u_ex,inf)/norm(f,1)))
disp(sprintf('  Ltb*: ||u_{ex}-u_{approx}||_{oo}/||f||_1=%d',norm(u3-u_ex,inf)/norm(f,1)))
