% XFFT - a Matlab toolbox to compute eXtended Fast Fourier transforms
% 
% Copyright (C) 2015 Ines Melzer
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
% 
% $LastChangedDate: 2015-10-21 14:16:54 +0200 (Mi, 21. Okt 2015) $
% $Author: melzer $
% $Revision: 6169 $
%
addpath('numerical_tests/utilities');


% 1 Laplace Transform
display('Laplace Transform')
M1=100;
M2=100;
epsilon=10^(-8);
MY=sort(5*rand(M1,1));
MOmega=sort(10*rand(M2,1));
n=1;
fhat=rand(M2,n);

q=8;
plan=XFLT('LT',MY,'accuracy',epsilon,'MOmega',MOmega);
f=mtimes(plan,fhat);
fprintf('Rang ,q=%d\n',q)
%fprintf('Anzahl der Gebiete ,  M=%d\n',M)

%u=exp(-MY*MOmega.')*fhat; %exact computation
u=zeros(M1,n);
for k=1:n
    u(:,k)=u_exactLT(MY,MOmega,fhat(:,k));
end

fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f-u,Inf)/norm(fhat,1))

pause

% 2 Combination of FFT-Laplace Transform
clear all;
display('--------------------------------------------------------')
display('FFLT')
M1=1000;
N=2^10;
epsilon=10^(-8);
MY=(linspace(0,10,M1)).';%sort(5*rand(M1,1));


n=1;
fhat=ones(M1,n);%rand(M1,n);
%fhat=zeros(M1,1);
%fhat(5)=1;

%u=exp(-MY*MOmega.')*fhat; %exact computation
u=zeros(M1,n);
for l=1:n
   u(:,l)=u_exactFFT(MY,fhat(:,l));
end


q=8;
plan=XFLT('FFLT',MY,'accuracy',epsilon);
f=mtimes(plan,fhat);

%plan_block=XFLT('FFLT',MY,'accuracy',epsilon,'blocksize',64);
%f2=mtimes(plan_block,fhat);
fprintf('Rang ,q=%d\n',q)
%fprintf('Anzahl der Gebiete ,  M=%d\n',M)
fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f-u,Inf)/norm(fhat(:,1),1))
%fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f2-u,Inf)/norm(fhat(:,1),1))

pause

% 3 Combination of NFFT-Laplace Transform
display('--------------------------------------------------------')
display('NFFLT')
clear all;
M1=1000;
N=2^10;
epsilon=10^(-8);
MY=sort(5*rand(M1,1));
MX=N*rand(M1,1);
n=1;

fhat=rand(N,1);

%u=exp(-MY*MOmega.')*fhat; %exact computation
MOmega=(0:1:N-1)';
u=zeros(M1,n);
for l=1:n
    u(:,l)=u_exactall(MY,MX,MOmega,N,fhat(:,l));
end

NFFTdir='/export/data/imelzer/software/nfft-3.3_optiplex20/';
q=8;
plan=XFLT('NFFLT',MY,'accuracy',epsilon,'N',N,'libdir',NFFTdir,'MX',MX);
f=mtimes(plan,fhat);

plan_block=XFLT('NFFLT',MY,'accuracy',epsilon,'N',N,'libdir',NFFTdir,'MX',MX,'blocksize',64);
f2=mtimes(plan_block,fhat);


fprintf('Rang ,q=%d\n',q)
%fprintf('Anzahl der Gebiete ,  M=%d\n',M)
%fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f-u,Inf)/norm(fhat,1))
display('ohne exakte Berechnung kleiner Blöcke')
fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f-u,Inf)/norm(fhat,1))
display('mit exakter Berechnung kleiner Blöcke')
fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f2-u,Inf)/norm(fhat,1))





% 4 Combination of BSFFT-Laplace Transform
clear all;
display('--------------------------------------------------------')
display('BSFFLT')
%M1=1000;
%M2=2000;

N=2^10;
M1=N;
M2=N;
epsilon=10^(-8);
MY=sort(5*rand(M1,1));
MX=N*rand(M1,1);
n=1;
%fhat=rand(M2,n);
fhat=ones(M2,1);
%f(2)=1;

%u=exp(-MY*MOmega.')*fhat; %exact computation

MOmega=N*sort(rand(M2,1));%(linspace(0,N,M2))';%N*sort(rand(M2,1));
u=zeros(M1,n);
for l=1:n
    u(:,l)=u_exactall(MY,MX,MOmega,N,fhat(:,l));
end
p=2;
q=2;
%profile on
%plan=XFLT('BSFFLT',MY,'accuracy',epsilon,'libdir',BSFFTdir,'MX',MX,'MOmega',MOmega,'N',N,'BSFFTrank',p);
plan=XFLT('BSFFLT',MY,'rank',q,'MX',MX,'MOmega',MOmega,'N',N,'BSFFTrank',p);
f=mtimes(plan,fhat);
%profile off
%profview




fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f(:,1)-u,Inf)/norm(fhat(:,1),1))
%fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f2-u,Inf)/norm(fhat(:,1),1))



% 5 Combination of NNFFT-Laplace Transform

display('--------------------------------------------------------')
display('NNFFLT')
M1=1000;
M2=2000;

N=2^10;
M1=N;
M2=N;
epsilon=10^(-8);
MY=sort(5*rand(M1,1));
MX=N*rand(M1,1);
n=1;
fhat=rand(M2,n);
fhat=ones(M2,1);
f(2)=1;

u=exp(-MY*MOmega.')*fhat; %exact computation
NNFFTdir='/home/math/imelzer/software/nfft-3.3_optiplex/matlab/';
MOmega=N*sort(rand(M2,1));%(linspace(0,N,M2))';%N*sort(rand(M2,1));
u=zeros(M1,n);
for l=1:n
    u(:,l)=u_exactall(MY,MX,MOmega,N,fhat(:,l));
end

q=8;

plan=XFLT('NNFFLT',MY,'rank',q,'libdir',NNFFTdir,'MX',MX,'MOmega',MOmega,'N',N);
f=mtimes(plan,fhat);

fprintf('Fehler ,||f-u||_oo/||f||_oo=%e\n',norm(f(:,1)-u,Inf)/norm(fhat(:,1),1))


